@@ -135,6 +135,7 @@ class MainWindow:
         self.content_type = TV_GROUP  # content being browsed
         self.back_page = None  # page to go back to if the back button is pressed
         self.active_channel = None
+        self.inhibit_id = 0
         self.fullscreen = False
         self.latest_search_bar_text = None
         self.visible_search_results = 0
@@ -145,6 +146,7 @@ class MainWindow:
 
         self.video_properties = {}
         self.audio_properties = {}
+        self.volume = 100
 
         # Used for redownloading timer
         self.reload_timeout_sec = 60 * 5
@@ -499,9 +501,6 @@ class MainWindow:
                     self.add_flag(COUNTRY_CODES[country_name], box)
                     break
 
-            if not found_flag:
-                print(f"No flag found for: {group.name}")
-
             for word in name.split():
                 self.add_badge(word, box, added_words)
 
@@ -509,7 +508,8 @@ class MainWindow:
             box.set_spacing(6)
             button.add(box)
             self.categories_flowbox.add(button)
-            self.categories_flowbox.show_all()
+
+        self.categories_flowbox.show_all()
 
         if not found_groups:
             self.on_category_button_clicked(None, None)
@@ -711,7 +711,7 @@ class MainWindow:
             surface = self.get_surface_for_file("/usr/share/hypnotix/generic_tv_logo.png", 22, 22)
         return surface
 
-    def on_go_back_button(self, widget):
+    def on_go_back_button(self, widget=None):
         self.navigate_to(self.back_page)
         if self.active_channel is not None:
             self.playback_bar.show()
@@ -734,20 +734,12 @@ class MainWindow:
             GLib.timeout_add_seconds(0.1, self.on_search)
 
     def on_search(self):
-        def filter_func(child):
-            search_bar_text = unidecode(self.search_bar.get_text()).lower()
-            label_text = unidecode(child.get_children()[0].get_children()[0].get_children()[1].get_text()).lower()
-            if search_bar_text in label_text:
-                self.visible_search_results += 1
-                return True
-            else:
-                return False
-
         self.visible_search_results = 0
-        self.channels_listbox.set_filter_func(filter_func)
-        if not self.channels_listbox.get_children():
-            self.show_channels(self.active_provider.channels)
-        print("Filtering %d channel names containing the string '%s'..." % (len(self.channels_listbox.get_children()), self.latest_search_bar_text))
+        channels = []
+        for channel in self.active_provider.channels:
+            if self.latest_search_bar_text in channel.name.lower():
+                channels.append(channel)
+        self.show_channels(channels)
         if self.visible_search_results == 0:
             self.status(_("No channels found"))
         else:
@@ -761,7 +753,6 @@ class MainWindow:
         self.active_group = None
         for child in self.channels_listbox.get_children():
             self.channels_listbox.remove(child)
-        self.channels_listbox.invalidate_filter()
         self.visible_search_results = 0
 
     @idle_function
@@ -964,6 +955,21 @@ class MainWindow:
         self.mpv.observe_property("audio-codec", self.on_audio_codec)
         self.mpv.observe_property("video-bitrate", self.on_bitrate)
         self.mpv.observe_property("audio-bitrate", self.on_bitrate)
+        self.mpv.observe_property("core-idle", self.on_playback_changed)
+
+    @idle_function
+    def on_playback_changed(self, prop, idle):
+        if idle:
+            if self.inhibit_id != 0:
+                self.application.uninhibit(self.inhibit_id)
+                self.inhibit_id = 0
+        else:
+            if self.inhibit_id == 0:
+                self.inhibit_id = self.application.inhibit(
+                    self.window,
+                    Gtk.ApplicationInhibitFlags.IDLE | Gtk.ApplicationInhibitFlags.SUSPEND,
+                    "Playing media"
+                )
 
     @idle_function
     def on_bitrate(self, prop, bitrate):
@@ -1527,9 +1533,17 @@ class MainWindow:
             else:
                 self.search_button.set_active(True)
         elif event.keyval == Gdk.KEY_F11 or \
-                (event.keyval == Gdk.KEY_f and not ctrl and type(widget.get_focus()) != gi.repository.Gtk.SearchEntry) or \
-                (self.fullscreen and event.keyval == Gdk.KEY_Escape):
-            self.toggle_fullscreen()
+                (event.keyval == Gdk.KEY_f and not ctrl and type(widget.get_focus()) != gi.repository.Gtk.SearchEntry):
+            self.full_screen_mode()
+        elif event.keyval == Gdk.KEY_F6:
+            self.theather_mode()
+        elif event.keyval == Gdk.KEY_F7:
+            self.borderless_mode()
+        elif event.keyval == Gdk.KEY_Escape:
+            self.normal_mode()
+        elif event.keyval == Gdk.KEY_BackSpace and not ctrl and type(widget.get_focus()) != gi.repository.Gtk.SearchEntry:
+            self.normal_mode()
+            self.on_go_back_button()
         elif event.keyval == Gdk.KEY_Left:
             self.on_prev_channel()
         elif event.keyval == Gdk.KEY_Right:
@@ -1591,6 +1605,7 @@ class MainWindow:
                         provider.password,
                         provider.url,
                         hide_adult_content=False,
+                        user_agent=self.settings.get_string("user-agent"),
                         cache_path=PROVIDERS_PATH,
                     )
                     if self.x.auth_data != {}:
@@ -1668,7 +1683,7 @@ class MainWindow:
             if ("=") in mpv_options:
                 pairs = mpv_options.split()
                 for pair in pairs:
-                    key, value = pair.split("=")
+                    key, value = pair.split("=", 1)
                     options[key] = value
         except Exception as e:
             print("Could not parse MPV options!")
@@ -1694,14 +1709,51 @@ class MainWindow:
             ytdl=True,
             wid=str(self.mpv_drawing_area.get_window().get_xid())
         )
+        self.mpv.volume = self.volume
+        self.mpv.observe_property("volume", self.on_volume_prop)
 
     def on_mpv_drawing_area_draw(self, widget, cr):
         cr.set_source_rgb(0.0, 0.0, 0.0)
         cr.paint()
 
-    def toggle_fullscreen(self):
+    def normal_mode(self):
+        self.window.unfullscreen()
+        self.mpv_top_box.show()
+        self.mpv_bottom_box.hide()
+        if self.content_type == TV_GROUP:
+            self.sidebar.show()
+        self.headerbar.show()
+        self.channels_box.set_border_width(12)
+        self.fullscreen = False
+
+    def theather_mode(self):
+        if self.stack.get_visible_child_name() == "channels_page":
+            if self.sidebar.get_visible():
+                self.mpv_top_box.hide()
+                self.mpv_bottom_box.hide()
+                self.sidebar.hide()
+                # self.headerbar.hide()
+                self.status_label.hide()
+                self.info_revealer.set_reveal_child(False)
+                self.channels_box.set_border_width(0)
+            else:
+                self.normal_mode()
+
+    def borderless_mode(self):
+        if self.stack.get_visible_child_name() == "channels_page":
+            if self.headerbar.get_visible():
+                self.mpv_top_box.hide()
+                self.mpv_bottom_box.hide()
+                self.sidebar.hide()
+                self.headerbar.hide()
+                self.status_label.hide()
+                self.info_revealer.set_reveal_child(False)
+                self.channels_box.set_border_width(0)
+            else:
+                self.normal_mode()
+
+    def full_screen_mode(self):
         if self.stack.get_visible_child_name() == "channels_page":
-            # Toggle state
             self.fullscreen = not self.fullscreen
             if self.fullscreen:
                 # Fullscreen mode
@@ -1714,21 +1766,16 @@ class MainWindow:
                 self.info_revealer.set_reveal_child(False)
                 self.channels_box.set_border_width(0)
             else:
-                # Normal mode
-                self.window.unfullscreen()
-                self.mpv_top_box.show()
-                self.mpv_bottom_box.hide()
-                if self.content_type == TV_GROUP:
-                    self.sidebar.show()
-                self.headerbar.show()
-                self.channels_box.set_border_width(12)
+                self.normal_mode()
 
     def on_fullscreen_button_clicked(self, widget):
-        self.toggle_fullscreen()
+        self.full_screen_mode()
 
     def on_close_info_window_button_clicked(self, widget):
         self.info_window.hide()
 
+    def on_volume_prop(self, name, value ):
+        self.volume = value
 
 if __name__ == "__main__":
     application = MyApplication("org.x.hypnotix", Gio.ApplicationFlags.FLAGS_NONE)
